#-*-coding:utf-8-*-
"""
    Interface de synchronisation entre le ldap et la bdd posh
    >>> from posh.api.scribe_api import ScribePosh
    >>> sync_handler = ScribePosh()
    Premier usage : synchronisation d'un groupe
    >>> sync_handler.synchronize_group('mongroupe')

    Second usage : synchronisation d'un type de groupe (Classe par défaut)
    >>> sync_handler.synchronize('Classe')

    Troisième usage : synchronisation forcée
                (recrée le groupe même si le cache pense qu'il y est déjà)
    >>> sync_handler.synchronize_group('mongroupe', force=True)
    >>> sync_handler.synchronize(force=True)
"""
import pickle
from os.path import isfile, isdir, join
from os import system

from scribe.eolegroup import Group
from scribe.eoleuser import User
from scribe.responsables import Responsable

from posh.logger.logger import getLogger
from posh.api.user_api import PoshUserApi
from posh.api.group_api import PoshGroupApi
from posh.api.usergroup_api import PoshUserGroupApi

log = getLogger("api.scribe_api")

SPEC_GROUPS = [] #['professeurs', 'eleves', 'administratifs']
TMP_DIR = "/usr/share/envole/tmp/"
GROUPIDS_FILE = join(TMP_DIR, "posh_groupids.pickle")
USERGROUPIDS_FILE = join(TMP_DIR, "posh_usergroupids.pickle")

def load_groupids(filename):
    """
        Charge les ids des groupes dans la base posh
        {nomdugroupe:{id:iddugroupe, parentid:iddugroupeparent(facultatif)},..}
    """
    if isfile(filename):
        return pickle.load(file(filename, 'r'))
    else:
        return {}

def save_groupids(filename, groupids_dict):
    """
        Sauvegarde les ids de groupe
    """
    pickle.dump(groupids_dict, file(filename, 'w'))
    return True

class UserSync(PoshUserApi):
    """
        Synchronise des/un utilisateurs
    """
    def __init__(self, scribeuser=User()):
        super(UserSync, self).__init__()
        self.scribeuser = scribeuser

    def synchronize_users(self):
        """
            Synchronise les utilisateurs de l'annuaire
        """
        log.info("#   Synchronisation des utilisateurs \
de l'annuaire   #")
        self.scribeuser.ldap_admin.connect()
        for user in self.scribeuser._get_users():
            self._create_user(user)
        self.scribeuser.ldap_admin.close()

    def synchronize_user(self, useruid):
        """
            Synchronise l'utilisateur useruid
            @useruid : useruid
        """
        self.scribeuser.ldap_admin.connect()
        self._create_user(useruid)
        self.scribeuser.ldap_admin.close()

    def _create_user(self, useruid):
        """
            Crée l'utilisateur useruid
            @useruid : uid de l'utilisateur
            @return : (id dans la bdd posh, @useruid)
        """
        try:
            longname = self.scribeuser._get_attr(useruid, 'displayName')[0]
        except IndexError:
            longname = useruid
        log.info("  + Création de l'utilisateur : %s" % (useruid,))
        return self.add_user(useruid, longname), useruid

class EnvoleMain(PoshUserApi):
    """
        Outil de synchronisation de base
    """
    def __init__(self):
        super(EnvoleMain, self).__init__()
        self.scribeuser = User()
        self.scribegroup = Group()
        self.scribeuser.ldap_admin = self.scribegroup.ldap_admin
        self.user_manager = UserSync(self.scribeuser)

    def purge(self, groupname=None):
        """
            Purge les utilisateurs de la base posh pour le groupe groupname
        """
        self.scribegroup.ldap_admin.connect()
        if groupname:
            users = self.scribegroup._get_members(groupname)
        else:
            users = self.scribeuser._get_users()
        for user in users:
            self.user_manager.del_user(user)
        self.scribegroup.ldap_admin.close()

class EnvoleUserGroups(EnvoleMain, PoshUserGroupApi):
    """
        Interface de synchronisation entre le ldap et la bdd posh
    """

    def __init__(self):
        super(EnvoleUserGroups, self).__init__()
        self.load_usergroupids()
        if not isdir(TMP_DIR):
            system("mkdir -p %s" % TMP_DIR)

# FONCTIONS PUBLIQUES (à utiliser)
    def synchronize_usergroups(self, _type='Classe', force=False):
        """
            Synchronise les groupes de type _type du ldap
            en tant que usergroup
            @_type : type de groupe Classe/Equipe/Niveau ...
            @force : forcer la synchronisation  (True/False)
        """
        log.info("#   Synchronisation des groupes de type : %s \
comme groupe utilisateur   #" % _type)
        self.scribegroup.ldap_admin.connect()
        for groupname in self.scribegroup._get_groups(_type=_type):
            self._synchronize_usergroup(groupname, force=force,
                                                    _type=_type)
        self.scribegroup.ldap_admin.close()
        self.save_usergroupids()

    def synchronize_usergroup(self, groupname, force=False):
        """
            Synchonise le groupe groupname comme groupe de carnet
            @groupname : nom du groupe
            @force : forcer la synchronisation  (True/False)
        """
        self.scribegroup.ldap_admin.connect()
        self._synchronize_usergroup(groupname, force=force)
        self.scribegroup.ldap_admin.close()
        self.save_usergroupids()

# FONCTIONS PRIVÉES
    def _synchronize_usergroup(self, groupname, force=False, _type=None):
        """
            Synchonise le groupe groupname (n'initialise pas
            la connexion ldap préférer synchronize_usergroup)
            le groupe sera nommé typedugroupe-nomdugroupe
        """
        print(" + Synchronisation du groupe %s + " % (groupname,))
        log.info(" + Synchronisation du groupe %s + " % (groupname,))
        if not _type:
            _type = self.scribegroup.c_get_group_type(groupname)
        if _type != "Base": # groupes professeurs, élèves, administratifs
            grname = "%s-%s" % (_type, groupname)
        else:
            grname = groupname.capitalize()
        groupid = self._create_usergroup(grname,
                                         force=force)
        self._populate_usergroup(groupname, groupid)
        return groupname, groupid

    def _create_usergroup(self, groupname, force=False):
        """
            Crée le groupe groupname comme groupe de carnet
        """
        if force or not self.usergroupids.has_key(groupname):
            log.info("  + Création du groupe : \"%s\"" % (
                                                           groupname,))
            # crée le groupe dans la base et enregistre l'id dans le dico
            usergroupid = self.add_usergroup(groupname, 'admin')
            self.usergroupids[groupname] = {'id':usergroupid}
            return usergroupid
        else:
            log.info("  + Le groupe \"%s\" existe déjà" % (
                                                           groupname,))
            return self.usergroupids[groupname]['id']

    def _populate_usergroup(self, groupname, groupid, synch_prof=True):
        """
            Peuple le groupe groupname
        """
        for user in self.scribegroup._get_members(groupname):
            userid = self.user_manager._create_user(user)[0]
            log.info("   + Inscription de l'utilisateur : %s" % (user,))
            self._subscribe_usergroup(groupid, userid)
        if synch_prof:
            if self.scribegroup._is_group('profs-%s' % groupname):
                for user in self.scribegroup._get_members('profs-%s' % (
                                                             groupname,)):
                    userid = self.user_manager._create_user(user)[0]
                    log.info("   + Inscription de l'enseignant : \"%s\"" % (
                                                                       user,))
                    self._subscribe_usergroup(groupid, userid)
    def save_usergroupids(self):
        """
            Sauve les ids des usergroups synchronisés
        """
        save_groupids(USERGROUPIDS_FILE, self.usergroupids)

    def load_usergroupids(self):
        """
            charge les ides des usergroups déjà synchronisé
        """
        self.usergroupids = load_groupids(USERGROUPIDS_FILE)

class EnvoleResponsables(EnvoleUserGroups):
    """
        Synchronise les responsables comme groupe de carnet (usergroups)
        Cette fonctionnalité est impossible par les autres outils (ref #1300)

        Pour faire évoluer la fonctionnalité (créer des groupes responsables-3eme)
        il faut faire évoluer le filtre qui permet de lister les utilisateurs.
    """
    def synchronize(self):
        self.scribeuser.ldap_admin.connect()
        self._synchronize()
        self.scribeuser.ldap_admin.close()

    def _synchronize(self):
        print(" + Synchronisation des responsables + ")
        log.info(" + Synchronisation des responsables + ")
        grname = "Responsables"
        groupid = self._create_usergroup(grname,
                                         force=True)
        self._populate_responsable(groupid)
        return grname, groupid

    def _populate_responsable(self, groupid):
        for user in self.scribeuser._get_users(filtre="(objectclass=responsable)"):
            userid = self.user_manager._create_user(user)[0]
            log.info("   + Inscription de l'utilisateur : %s" % (user,))
            self._subscribe_usergroup(groupid, userid)


class EnvoleProfile(EnvoleMain, PoshGroupApi):
    """
        Interface de synchronisation entre le ldap
        et les groupes de profil posh
    """

    def __init__(self):
        super(EnvoleProfile, self).__init__()
        self.scribeuser = User()
        self.scribegroup = Group()
        self.scribeuser.ldap_admin = self.scribegroup.ldap_admin
        self.user_manager = UserSync(self.scribeuser)
#        self.load_usergroupids()
#        self.load_groupids()
#        if not isdir(TMP_DIR):
#            system("mkdir -p %s" % TMP_DIR)

# FONCTIONS PUBLIQUES (à utiliser)
    def envole_synchronize(self):
        """
            Synchronise les groupes de l'annuaire
            selon : http://dev-eole.ac-dijon.fr/issues/635
        """
        self.scribegroup.ldap_admin.connect()
        self.all_users = dict()#[(login, set())
                               #for login in self.scribeuser._get_users()])
        self.envole_synchronize_eleve()
        self.envole_synchronize_prof()
        self.envole_synchronize_equipe()
        self.envole_synchronize_responsables()
        self.envole_synchronize_personnels()
        self.scribegroup.ldap_admin.close()
        self.envole_populate()

    def envole_synchronize_eleve(self):
        """
            Synchronise:
            + eleves
                |_ niveaux
                     |_ classes
        """
        log.info(" + Synchronisation des élèves")
        eleve_groupid = self._create_group("eleves")
        structure = self.scribegroup._get_classes()
        niveaux = dict([("niveau-%s" % niv, {})for niv in structure])
        for niv, cls in structure.items():
            niv_name = "niveau-%s" % niv
            niv_id = self._create_group(niv_name,
                                        group_parent_id=eleve_groupid)
            niveaux[niv_name]['id'] = niv_id
            niveaux[niv_name]['cls'] = {}
            for cl in cls:
                gr_name = "classe-%s" % cl
                gr_id = self._create_group(gr_name,
                                           group_parent_id=niv_id)
                self._envole_subscribe_users(cl, gr_id)
                niveaux[niv_name]['cls'][gr_name] = gr_id
        return dict(eleves=niveaux)

    def envole_synchronize_prof(self):
        """
            Synchronise:
            + professeurs
                   |_ disciplines
        """
        log.info(" + Synchronisation des enseignants")
        prof_groupid = self._create_group("professeurs")
        structures = self.scribegroup._get_groups('Matiere')
        matieres = dict([(mat, {})for mat in structures])
        for mat in structures:
            gr_name = "matière-%s" % mat
            gr_id = self._create_group(gr_name,
                                        group_parent_id=prof_groupid)
            self._envole_subscribe_users(mat, gr_id)
            matieres[mat]['id'] = gr_id
        return dict(professeurs=matieres)

    def envole_synchronize_equipe(self):
        """
            Synchronise  :
            + equipes pedagogiques
                   |_ classes
        """
        log.info(" + Synchronisation des équipes pédagogiques")
        equipe_groupid = self._create_group("équipes pédagogiques")
        structures = self.scribegroup._get_groups('Equipe')
        equipes = dict([(eq, {})for eq in structures])
        for eq in structures:
            eq_id = self._create_group("%s" % (eq,),
                                        group_parent_id=equipe_groupid)
            self._envole_subscribe_users(eq, eq_id)
            equipes[eq]["id"] = eq_id
        return dict(equipes=equipes)

    def envole_synchronize_responsables(self):
        """
            Synchronise :
            + responsables
        """
        log.info(" + Synchronisation des responsables")
        resp_id = self._create_group("responsables")
        scribe_resp = Responsable()
        scribe_resp.ldap_admin = self.scribeuser.ldap_admin
        self._envole_subscribe_resps(resp_id)
        return dict(responsables=dict(id=resp_id))

    def envole_synchronize_personnels(self):
        """
            Synchronise :
            + personnels
                   |_ administration
                   |_ education
        """
        log.info(" + Synchronisation des personnels de l'établissement")
        administratif_id = self._create_group("personnels")
        administration_id = self._create_group("administration",
                                               group_parent_id=administratif_id)
        self._envole_subscribe_users("administratifs", administration_id)
        education_id = self._create_group("education",
                                          group_parent_id=administratif_id)
        self._envole_subscribe_users("professeurs", education_id)
        return dict(personnels=dict(id=administratif_id,
                                    cls=dict(administration=administration_id,
                                             education=education_id)))

    def _envole_subscribe_users(self, groupname, groupid):
        """
            Stocke l'id du groupe
            pour une inscription future de ses utilisateurs
        """
        for user in self.scribegroup._get_members(groupname):
            self.user_manager._create_user(user)
            log.info("   + Ajout d'une référence au groupe : %s pour \
l'utilisateur : %s" % (groupname, user))
            self.all_users.setdefault(user, set()).add(groupid)

    def _envole_subscribe_resps(self, groupid):
        """
            Stocke l'id du groupe responsables
            pour une inscription future de ses utilisateurs
        """
        resp = Responsable()
        resp.ldap_admin = self.scribeuser.ldap_admin
        for user in resp._get_responsables():
            self.user_manager._create_user(user)
            log.info("   + Ajout d'une référence au groupe responsable \
pour l'utilisateur : %s" % (user,))
            self.all_users.setdefault(user, set()).add(groupid)

    def envole_populate(self):
        """
            Crée les comptes utilisateurs et les inscrits dans les groupes
        """
        for user, groupids in self.all_users.items():
            log.debug(" + Inscription de l'utilisateur %s à ses groupes" % (
                                                                    user,))
            self.subscribe_profilgroups_by_groupids(user, list(groupids))

# Fonction de base
    def _create_group(self, groupname, group_parent_id=0):
        """
            Crée le groupe groupname comme groupe de profile
            @groupname : le nom du groupe
            @group_parent_id : l'id du groupe parent

            @return : l'id du group (groupname)
        """
        log.info("  + Création du groupe : \"%s\"" % (
                                              groupname,))
        # crée le groupe dans la base et enregistre l'id dans le dic
        return self.add_group(groupname,
                              group_parent_id=group_parent_id)
