# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# modeles d'onglets
#
###########################################################################

"""
Modeles d'onglets (thumbnails)
"""

def filterunicode(value):
    """
        Filtre pour les données non alphanumériques
        typiquement celles passées au sein d'un tube xmlrpc
    """
    if isinstance(value, unicode):
        return value.encode('utf-8')
    else:
        return value

class Rule:
    """Définition d'un ensemble de règles sur des onglets
    (pour l'instant, authorisation, interdiction)
    """
    def __init__(self):
        """
        thumbs : ensemble des onglets autorisés par la règle
        """
        self.thumbs = set()
        self.desktop_items = set()

    def add_thumb(self, thumb):
        """
        thumb : objet onglet
        True: authorisation
        False: interdiction de l'onglet
        """
        if thumb not in self.thumbs:
            self.thumbs.add(thumb)

    def add_thumbs(self, thumbs):
        """methode pratique pour entrer une liste de règles"""
        for thumb in thumbs:
            self.add_thumb(thumb)

    def add_desktop_item(self, desktop_item):
        """ méthode pour ajouter un item de bureau à la règle
            desktop_item: objet bouton du bureau
        """
        if desktop_item not in self.desktop_items:
            self.desktop_items.add(desktop_item)

    def add_desktop_items(self, desktop_items):
        """ methode pour rentrer des item"""
        for desktop_item in desktop_items:
            self.add_desktop_item(desktop_item)

class Thumb:
    """
        Modelisation d'un onglet portaneo
    """

    def __init__(self, name, description= "pas de description", seq=None,
                          _type=None, param=None, admin_profile_id='-1'):
        """
        description: **note** : la description n'est pas présente dans la table
        name: nom affiche dans l'onglet
        seq: ordre d'affichage de l'onglet
        _type: type d'onglet (html:2, page personnalisable:1, javascript:3, redirection:4)
        param: url de l'onglet
        profile_id : id de la page dans la table profil posh pour l'utilisateur admin
                    (utilisé pour retrouver les widgets associés )
        """
        self.description = filterunicode(description)
        self.name = filterunicode(name)
        self.seq = str(seq).strip()
        if _type:
            self._type = str(_type).strip()
        else:
            self._type = "2"
        if param:
            self.param = param.strip()
        else:
            self.param = None
        if str(self._type) == '2':
            self.admin_profile_id = '-1'
        else:
            self.admin_profile_id = admin_profile_id

    def __str__(self):
        return self.name

    def to_dbfield(self):
        """
            renvoie une liste de données pour insertion dans la base posh
            xmlrpc: True si le résultat dout passer par de l'xmlrpc
        """
        # reformons un champ de la base avec l'objet thumb
        return [self.name,
                self.description,
                self.seq,
                self._type,
                self.param or '',
                self.admin_profile_id]

    def __hash__(self):
        if self.param != None:
            return hash(self.param)
        else:
            return hash(self.name)

    def __eq__(self, other):
        if self.param not in [None, 'None']:
            return self.param == other.param
        elif self.name:
            return self.name == other.name
        else:
            return self.description == other.description

    def encoding(self, value):
        try:
            return value.encode('utf-8')
        except:
            return str(value)

class DesktopItem:
    """
        Modélisation d'un item de bureau portaneo
    """
    TEMPLATE = """<buttondata><icon><![CDATA[%(icon)s]]></icon>\
<libelle><![CDATA[%(libelle)s]]></libelle><url><![CDATA[%(url)s]]></url>\
</buttondata>"""

    def __init__(self, name, libelle, url, icon='', method=''):
        """
            name:nom unique de l'item de bureau
            libelle:libellé du bouton sur le bureau
            url:url à charger
            icon:icone associée au bouton
            method:méthode utilisée pour l'ouverture de la page
                            (sert à rien pour l'instant mais pourrait
                              être un window.open ou un truc du genre)
        """
        self.name = filterunicode(name)
        self.libelle = filterunicode(libelle)
        self.url = url
        self.icon = icon
        self.method = method

    def __str__(self):
        return "%s --> %s" % (self.name, self.url)

    def __eq__(self, other):
        return self.name == other

    def __hash__(self):
        return hash(self.name)

    def toxml(self):
        """
            méthode appellée depuis l'extérieur
        """
        return self.TEMPLATE % dict(libelle=self.libelle,
                            icon=self.icon, url=self.url)

class Exporter:
    """
        Droit à l'exportation d'onglet
    """
    def __init__(self, group_ids):
        """
            :group_ids: ids des groupes auxquels l'utilisateur
                        a le droit d'ajouter des onglets
        """
        self.group_ids = group_ids

    def __eq__(self, other):
        """
            Teste l'égalité de deux objets exporter
        """
        return set(self.group_ids) == set(other.group_ids)

    def toxml(self):
        """
            Mise en forme xml de la permission
        """
        from xml.etree.ElementTree import Element, SubElement, tostring
        groups = Element('groups')
        for num_groupe in self.group_ids:
            groupelement = SubElement(groups, "group")
            groupelement.text = str(num_groupe)
        return tostring(groups)

class Module:
    """
        Modélisation d'un widget portaneo
    """
    def __init__(self, argument_list):
        """
            argument_list : entrée de bdd d'un widget dans
            la table module de posh (sous forme de liste)
        """
        self.arguments = argument_list

    def __eq__(self, other):
        """ comparaison sur
            - id_module (type de widgets),
            - la variable (pour les flux rss, les carnets...)
            - identifiant du module dans la page
        """
        return (self.arguments[0],
                self.arguments[9],
                self.arguments[10]) == other

    def __hash__(self):
        return hash((self.arguments[0],
                     self.arguments[9],
                     self.arguments[10]))

    def __repr__(self):
        return "%s ---- %s ---- %s ---- %s" % (self.arguments[0],
                                               self.arguments[2],
                                               self.arguments[9],
                                               self.arguments[10])

