# -*- coding: UTF-8 -*-
"""
    Outil d'authentification
    PamAuth permet l'auth PAM
    AuthenticatedPage est une page Nevow garantissant l'authentification:
    class Page(AuthenticatedPage):
        ...
        def locatechild(self, ctx, segments):
            session = inevow.ISession(ctx)
            request = inevow.IRequest(ctx)
            # authentification
            root_url = _get_root_url(ctx)
            togo = self.authenticated(request, session, root_url)
            if togo:
               return self.auth_redirect(togo)
        ...
"""
import PAM
from time import time
import xmlrpclib
from os.path import join, isfile

from twisted.web.resource import Resource
from twisted.internet.task import LoopingCall

from nevow.rend import Page
try:
    from posh.config.config import SESSION_TIMEOUT, AUTHFILE_PATH
    from posh.logger.logger import getLogger
    log = getLogger("interface.auth")
except:
    from twisted.python import log
    log.error = log.err
    log.info = log.debug = log.msg
    #timeout de la session de l'interface en 12 min si pas d'action
    SESSION_TIMEOUT = 12 * 60

#fréquence de vérification de la session
LIFETIME = 60

def get_authorized_users():
    """ renvoie une liste des utilisateurs autorisés
        à utiliser l'interface d'administration
    """
    f_name = join(AUTHFILE_PATH, "auth_admin_profil.txt")
    if isfile(f_name):
        users = file(f_name, 'r').read().splitlines()
        users.append('admin')
        return users
    return ['admin']

autorized_users = get_authorized_users()

class PamAuth():
    """AUTHENTIFICATION PAM
    """
    def __init__(self, service='passwd'):
        self.service = service
        self.auth = PAM.pam()

    def pam_conv(self, auth, query_list, userData):
        """fonction de dialogue avec la lib_pam"""
        resp = []
        for i in range(len(query_list)):
            query, _type = query_list[i]
            if _type == PAM.PAM_PROMPT_ECHO_ON:
                val = raw_input(query)
                resp.append((val, 0))
            elif _type == PAM.PAM_PROMPT_ECHO_OFF:
                resp.append((self.passwd, 0))
            elif _type == PAM.PAM_PROMPT_ERROR_MSG or \
                 _type == PAM.PAM_PROMPT_TEXT_INFO:
                resp.append(('', 0))
            else:
                return None
        return resp

    def authenticate(self, user, passwd):
        self.auth.start(self.service)
        self.passwd = passwd
        if user != None:
            self.auth.set_item(PAM.PAM_USER, user)
        self.auth.set_item(PAM.PAM_CONV, self.pam_conv)
        try:
            self.auth.authenticate()
            self.auth.acct_mgmt()
            self.passwd = None
        except PAM.error:
            return False
        except:
            log.error('Internal error')
            return False
        else:
            return True


class SessionTimer:
    """
       timer de session nevow
       session : objet ISession nevow
       timeout : timeout de la session
       lifetime : fréquence de contrôle
    """
    def __init__(self, session, timeout=SESSION_TIMEOUT, lifetime=LIFETIME):
        self.session = session
        self.sessionTimeout = timeout
        self.touch()
        self.looping = LoopingCall(self.check_expiration)
        self.looping.start(lifetime, now=False)

    def touch(self):
        """ remet à jour le timer
            une action a été effectuée
        """
        self.lastModified = self._getTime()

    def _getTime(self):
        """ renvoie le temps courant
        """
        return time()

    def check_expiration(self):
        """
            teste l'expiration de la session
        """
        log.debug("Check session expiration")
        if self._getTime() - self.lastModified > self.sessionTimeout:
            self.looping.stop()
            self.session.kill_myself(self.session)

class Redirection(Resource):
    """
        Objet de redirection:
            redirection html
            redirection ajax : se contente de renvoyer la chaine 'refresh'
                                (géré par le js client (handleResponse de 1_ajax.js)
            @_type : type de redirection (ajax ou autre chose pour html)
            @url : url vers laquelle redirigé
    """
    def __init__(self, _type, url):
        Resource.__init__(self)
        self.url = url
        self._type = _type

    def redirect_html(self, request):
        log.debug("Redirection to %s" % self.url)
        request.redirect(self.url)
        return """<html> \
<head><meta http-equiv=\"refresh\" content=\"0;URL=%(url)s\"> \
</head> \
<body bgcolor=\"#FFFFFF\" text=\"#000000\"> \
<a href=\"%(url)s\">click here</a> \
</body></html> \
""" % {'url': self.url}

    def redirect_ajax(self):
        """ redirection pour les requêtes ajax """
        log.debug("Ajax redirection to rootpage")
        return """refresh"""

    def render(self, request):
        if self._type == "ajax":
            return self.redirect_ajax()
        else:
            return self.redirect_html(request)

    def getChild(self, name, request):
        return self


class AuthenticatedPage(Page):
    """
        Page authentifiée
        auth_form : adresse du formulaire d'authentification
        auth_server : adresse du serveur xmlrpc d'authentification

        class MyPage(AuthenticatedPage):
            ...
            def locatechild(self, ctx, segments):
                session = inevow.ISession(ctx)
                request = inevow.IRequest(ctx)
                # authentification
                root_url = _get_root_url(ctx)
                togo = self.authenticated(request, session, root_url)
                if togo:
                   return self.auth_redirect(togo)
            ...

        gère un timer de session, les valeurs sont initialisées en dur dans le code
    """

    def __init__(self, auth_form, auth_server):
        self.auth_server = auth_server
        self.auth_form = auth_form
        super(AuthenticatedPage, self).__init__()

    def _init_ssoparams(self, session):
        """ renvoie le client xmlrpc pour la connexion au sso
        """
        def kill_myself(sess):
            log.debug("Session Timed out: killing the session")
            delattr(session, 'timer')
            sess.expire()

        if not hasattr(session, "sso"):
            session.sso = xmlrpclib.ServerProxy(join(self.auth_server,
                                                            'xmlrpc'))
        if not hasattr(session, "is_logged_in"):
            session.is_logged_in = None
        # timer de session
        if not hasattr(session, "kill_myself"):
            session.kill_myself = kill_myself
        if not hasattr(session, "timer"):
            log.debug("Init session Timer")
            session.timer = SessionTimer(session)
        else:
            log.debug("Refreshing the session timer")
            session.timer.touch()

    def reset_session(self, session):
        if hasattr(session, 'is_logged_in'):
            delattr(session, 'is_logged_in')
        if hasattr(session, 'timer'):
            session.timer.looping.stop()
        if hasattr(session, 'kill_myself'):
            session.kill_myself(session)
            delattr(session, "kill_myself")

    def auth_redirect(self, redirection):
        """ renvoie la redirection vers redirection
        """
        return redirection

    def _get_request_type(self, request):
        """ renvoie le type de requête à laquelle il faut répondre
            si la requete à un x-requested-with de valeur XMLHttpRequest dans son header:
                c'est une requête ajax,
            sinon il s'agit d'une requête html classique
        """
        if request.getHeader('x-requested-with') == "XMLHttpRequest":
            log.debug("It's an ajax request")
            return "ajax"
        else:
            log.debug("It's an html request")
            return 'html'

    def is_autorized_user(self, uid):
        """ valide l'authentification par login
        """
        if uid in autorized_users:
            return True
        log.error(" - Erreur: utilisateur non autorisé")
        return False

    def authenticated(self, request, session, root_url):
        """
            authentifie la session de l'utilisateur, 3 situations:
            1 - la session est authentifiée
            2 - on a un ticket , on récupère les infos utilisateurs,
                on valide les droits et on marque la session comme authentifiée
            3 - on redirige vers le sso pour authentification
        """
        self._init_ssoparams(session)
        redirect_type = self._get_request_type(request)
        # 1 -
        if session.is_logged_in:
            log.debug(" + Session authentifiée pour %s" % session.is_logged_in)
            return None
        # 2 -
        if request.args.has_key('ticket'):
            log.debug(" + On a un ticket SSO ")
            app_ticket = request.args['ticket'][0]
            datas = session.sso.get_user_details(app_ticket, root_url)
            if datas[0]:
                uid = datas[1]['uid'][0]
                if self.is_autorized_user(uid) :
                    session.is_logged_in = uid
                    log.info("* Connexion de l'utilisateur : %s *" % uid)
                    log.debug("===> Le ticket d'application a été validé, \
l'utilisateur %s est autorisé à accéder à \
l'application." % uid)
                    return Redirection(redirect_type,
                                        root_url), ()
                else:
                    return Redirection(redirect_type,
                                        join(self.auth_form, 'logout')), ()
        # 3 -
        log.debug(" - On est pas authentifié")
        return Redirection(redirect_type,
                            join(self.auth_form, '?service=%s' % root_url)), ()
